package com.brilliance.isc.funds.common.component;

import com.brilliance.isc.bo.Cbb;
import com.brilliance.isc.bo.Cnd;
import com.brilliance.isc.bo.Ftd;
import com.brilliance.isc.bo.model.*;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.component.PtsptaToolComponent;
import com.brilliance.isc.common.exception.CommonServiceException;
import com.brilliance.isc.common.sysmod.SysmodService;import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.util.Utils;
import com.brilliance.isc.common.vo.PtsptaVo;

import com.brilliance.isc.mda.dao.FtdselMapper;
import com.brilliance.isc.mda.dao.CbbMapper;
import com.brilliance.isc.mda.dao.CndMapper;

import com.brilliance.isc.vo.TrncodVo;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import com.brilliance.mda.runtime.mda.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

@Component
public class FtToolComponent {


    //        @Autowired
//        private CpdMapper cpdMapper;
    @Autowired
    private TrnmodService trnmodService;
    @Autowired
    private FtdselMapper ftdselMapper;
    @Autowired
    private CbbMapper cbbMapper;
    @Autowired
    private CbsmodService cbsmodService;
    @Autowired
    private PtsptaToolComponent ptsptaToolComponent;
    @Autowired
    private CndMapper cndMapper;
    @Autowired
    private SysmodService sysmodService;

    public Ftdgrp getFtdgrpByInr(String inr) {

        Ftd ftd = new Ftd();
        ftd.setInr(inr);
        ftd = ftdselMapper.selectByPrimaryKey(ftd);
        if (ftd == null) {
            throw new CommonServiceException("主键" + inr + "找不到FTD数据");
        }
        Ftdgrp ftdgrp = new Ftdgrp();
        ftdgrp.reset();
        ftdgrp.setRec(ftd);
        return ftdgrp;
    }

       public void getref(Ftdgrp ftdgrp) {
            //! Set reference number, if not already set (usually called in transaction`s MTABUT\\SAV Event)
            String tbl = NULLSTR;

            switch (ftdgrp.getRec().getFttyp()) {
                case "PT":
                    tbl = "PT";
                    break;
                    /**VP-同存调出
                       VN-同存户转
                    **/
                case "VP":
                    tbl = ftdgrp.getRec().getFttyp();
                    break;
                default:
                    tbl = "FT";
            }
            if (MdaUtils.isEmpty(ftdgrp.getRec().getOwnref())) {

                ftdgrp.getRec().setOwnref(sysmodService.getRef(tbl, 5));
            }

        }

    public void getCndInfo(Cpdgrp cpdgrp, String objInr, String objType) {
        List<Cnd> cnds = cndMapper.selectByObjtypAndObjinr(objType, objInr);
        if (!MdaUtils.isEmpty(cnds)) {
            cpdgrp.setPyebchkey(cnds.get(0).getPyebchkey());
            cpdgrp.setPyebchnam(cnds.get(0).getPyebchnam());
        }
    }

    public void storePtsToGrp(Object grpObject, String objInr, String objType) {
        //找ptspta数据
        Map<String, PtsptaVo> maps = ptsptaToolComponent.getPtsptaList(objInr, objType);
        if (maps.size() > 0) {
            maps.keySet().forEach(key -> {
                PtsptaVo ptsptaVo = maps.get(key);
                if (key.equalsIgnoreCase("INT")) {
                    key = "tmpint";
                }
                Utils.setFieldValue(grpObject, key.toLowerCase(), ptsptaVo);
            });
        }
    }

    public void collectCbsMap(TransactionServiceSet set, String prefix, Ftacbs cbs, String... cbsNames) {
        Map<String, Cbb> cbsMap = set.getCbsMap();
        if (cbsMap == null) {
            cbsMap = new HashMap<>();
        }
        for (String name : cbsNames) {
            Cbb cbb = Utils.getFieldValue(cbs, name);
            if (cbb != null) {
                cbsMap.put(prefix.toUpperCase() + name.toUpperCase(), cbb);
            }
        }
        set.setCbsMap(cbsMap);
    }


    public void collectPtsList(TransactionServiceSet set, Ftdgrp ftdgrp, String... ptsNames) {
        if (ptsNames == null) {
            //找所有的ptspta对象
            Field[] fields = ftdgrp.getClass().getDeclaredFields();
            for (Field field : fields) {
                field.setAccessible(true);
                if (PtsptaVo.class.equals(field.getType())) {
                    PtsptaVo ptspta = Utils.getFieldValue(ftdgrp, field.getName());
                    if (ptspta != null) {
                        set.addPtspta(transferFrom(ptspta));
                    }
                }
            }
        } else {
            for (String name : ptsNames) {
                PtsptaVo ptspta = Utils.getFieldValue(ftdgrp, name);
                if (ptspta != null && ptspta.getPts() != null && StringUtils.isNotEmpty(ptspta.getPts().getRol())) {
                    set.addPtspta(transferFrom(ptspta));
                }
            }
        }
    }



    /**
     * 根据STL结构的ptspta转化成抽象组件的ptspta，结构不一样
     *
     * @param ptsptas
     * @return List<com.brilliance.common.vo.PtsptaVo>
     */
    protected PtsptaVo transferFrom(PtsptaVo ptsptas) {
        PtsptaVo ptspta = ptsptas;
        ptspta.setRol(ptsptas.getPts().getRol());
        ptspta.setPtsinr(ptsptas.getPts().getInr());
        ptspta.setNam(ptsptas.getPts().getNam());
        ptspta.setPtyinr(ptsptas.getPts().getPtyinr());
        ptspta.setPtainr(ptsptas.getPts().getPtainr());
        ptspta.setExtkey(ptsptas.getPts().getExtkey());
        ptspta.setDftdsp(ptsptas.getPts().getDftdsp());
        ptspta.setDftcur(ptsptas.getPts().getDftcur());
        ptspta.setDftact(ptsptas.getPts().getDftact());
        ptspta.setDftfeecur(ptsptas.getPts().getDftfeecur());
        ptspta.setDftactptainr(ptsptas.getPts().getDftactptainr());
        ptspta.setDftselrol(ptsptas.getPts().getDftsetrol());
        ptspta.setGlggrpflg(ptsptas.getPts().getGlggrpflg());
        ptspta.setRef(ptsptas.getPts().getRef());
        ptspta.setPtytyp(ptsptas.getPtytyp());
        ptspta.setPtyextkey(ptsptas.getPtyextkey());
        ptspta.setAdrblk(ptsptas.getPts().getAdrblk());
        return ptspta;
    }

    public static void isFTTrnAllowed(Ftdgrp ftdgrp, TrncodVo trncodVo) {
        ftdgrp.reset();
        String trnNam = trncodVo.getInifrm().toUpperCase();
        String errOpnY = "Selected foreign exchange is already opened.";
        String errOpnN = "No foreign exchange selected.";
        String errClsY = "Selected foreign exchange has been closed.";
        String errQuoY = "Selected foreign exchange has been quoted.";
        String errQuoN = "Selected foreign exchange has not quoted yet.";
        String errSigY = "Sight exchange cannot settle at maturity.";
        String errCnfY = "Selected foreign exchange has been confirmed.";
        String errCnfN = "Selected foreign exchange has not been confirmed.";
        String errAccN = "There is no amount in transit.";
        String errZJB  = "此交易只能资金部经办！";
        String errYYB  = "此交易只能营运部经办！";
        String errHO   = "Only Headoffice can quote.";
        String errRel  = "The former registration transaction(FXTSOP/FXTBOP) not released yet.";
        String errSetY = "Selected foreign exchange has been settled.";
        String errLB = "";

        if ("FXTLOP".equals(trnNam)||"FXTFOP".equals(trnNam)||"FXTDEP".equals(trnNam)) {

            if (!StringUtils.isEmpty(ftdgrp.getRec().getInr())){
                trncodVo.setDisResult(errOpnY);
                trncodVo.setIsAllowed("N");
            }
        }

        if ("FXTLCM".equals(trnNam)||"FXTFCM".equals(trnNam)||"FXTDEM".equals(trnNam)) {

            if (StringUtils.isEmpty(ftdgrp.getRec().getInr())){
                trncodVo.setDisResult(errOpnN);
                trncodVo.setIsAllowed("N");
            }else if(Objects.nonNull(ftdgrp.getRec().getCnfdat())){
                trncodVo.setDisResult(errCnfY);
                trncodVo.setIsAllowed("N");
            }

            if(Objects.nonNull(ftdgrp.getRec().getClsdat())){
                trncodVo.setDisResult(errClsY);
                trncodVo.setIsAllowed("N");
            }

        }


        if ("FXTLCN".equals(trnNam)||"FXTSUC".equals(trnNam)||"FXTFCN".equals(trnNam)||"FXTDEN".equals(trnNam)) {
            if (StringUtils.isEmpty(ftdgrp.getRec().getInr())){
                trncodVo.setDisResult(errOpnN);
                trncodVo.setIsAllowed("N");
            }else if(ftdgrp.getCbs().getOpn2().getAmt().compareTo(BigDecimal.valueOf(0)) <= 0){
                trncodVo.setDisResult(errAccN);
                trncodVo.setIsAllowed("N");
            }

            if(Objects.nonNull(ftdgrp.getRec().getClsdat())){
                trncodVo.setDisResult(errClsY);
                trncodVo.setIsAllowed("N");
            }

            if(Objects.isNull(ftdgrp.getRec().getCnfdat())){
                trncodVo.setDisResult(errCnfN);
                trncodVo.setIsAllowed("N");
            }

        }

        if(MdaUtils.isEmpty(trnNam)) {
            String msgTxt = "The selected transaction cannot be started under the selected F/X.";
            trncodVo.setDisResult(msgTxt);
            trncodVo.setIsAllowed("N");
        }
        if(Strings.isEmpty(trncodVo.getDisResult())){
            trncodVo.setIsAllowed("Y");
        }

    }
        public void sav(Ftdgrp ftdgrp) {
                    trnmodService.trnStoreContract(ftdgrp);
        }
}


