package com.brilliance.isc.funds.fxtssb.service;

import com.brilliance.isc.bo.*;
import com.brilliance.isc.bo.model.Fxacbs;
import com.brilliance.isc.bo.model.Fxdgrp;
import com.brilliance.isc.common.api.bch.service.BchService;
import com.brilliance.isc.common.cache.pta.PtaCacheService;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.cbsmod.service.XrtmodService;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.glemod.service.GlemodService;
import com.brilliance.isc.common.setmod.service.SetmodService;
import com.brilliance.isc.common.setmod.vo.SetmodVo;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.I18nUtil;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.funds.bo.funds.FxtssbStoreBo;
import com.brilliance.isc.funds.common.component.FxToolComponent;
import com.brilliance.isc.funds.fxtlop.service.FxtpService;
import com.brilliance.isc.funds.fxtssb.check.FxtssbValidator;
import com.brilliance.isc.funds.vo.funds.FxtssbStoreVo;
import com.brilliance.isc.mda.dao.PtyratMapper;
import com.brilliance.isc.vo.funds.FxdBizInfoVo;
import com.brilliance.mda.runtime.mda.IStream;
import com.brilliance.mda.runtime.mda.impl.Argument;
import com.brilliance.mda.runtime.mda.impl.StreamImpl;
import com.brilliance.mda.runtime.mda.util.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

/**
 * @program: isc-funds
 * @ClassName: FxtssbTransactionServiceImpl
 * @description: 即期结汇 Fxtssb 业务处理
 * @author: huangshunlin
 * @create: 2024-10-08 15:35
 */
@Service("fxtssb.transaction")
public class FxtssbTransactionServiceImpl extends AbstractTransactionService<FxtssbStoreBo> {

    @Resource
    private FxtssbValidator fxtssbValidator;

    @Resource
    private XrtmodService xrtmodService;

    @Resource
    private SysmodService sysmodService;

    @Resource
    private FxToolComponent fxToolComponent;


    @Resource
    private TransactionHelpService transactionHelpService;


    @Resource
    private SettleContext settleContext;

    @Resource
    private CbsmodService cbsmodService;

    @Resource
    private TrnmodService trnmodService;

    @Resource
    private GlemodService glemodService;

    @Autowired
    private PtaCacheService ptaCacheService;

    @Resource
    private BchService bchService;

    @Resource
    private PtyratMapper ptyratMapper;

    @Resource
    private SetmodService setmodService;

    @Resource
    private FxtpService fxtpService;


    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "N");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_JSH_SERVICE, "Y");
    }


    @Override
    protected void txnSave(FxtssbStoreBo transactionVo) {
        ruleMtabutSavN500(transactionVo);
    }

    public void ruleMtabutSavN500(FxtssbStoreBo bo) {
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();
        Fxdgrp fxdgrp = bo.getFxdgrp();
        fxToolComponent.getref(fxdgrp);
        // 1. Create (rsp. update) Contract (incl. Texts)
        Argument<String> pathBox = new Argument<String>("", fxdgrp.getRec().getAccmng());
        Argument<String> pathBox1 = new Argument<String>("", fxdgrp.getRec().getHdbch());
        sysmodService.getPtyAccMng(fxdgrp.getApl().getPta().getPtyinr(), "", "", "", pathBox, pathBox1);

        fxdgrp.getRec().setAccmng(pathBox.value);
        fxdgrp.getRec().setHdbch(pathBox1.value);
        fxToolComponent.sav(fxdgrp, BigDecimal.ZERO);
        // 2. Build Transaction from FTDGRP
        trnmodService.trnSetFromContract(trnmodVo);

        cbsSav(bo, trnmodVo);
        // save documents

    }

    @Override
    protected Map<String, String> txnCheck(FxtssbStoreBo transactionVo) {
        return fxtssbValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(FxtssbStoreBo transactionVo) {
        FxdBizInfoVo bizInfoVo = new FxdBizInfoVo();
        bizInfoVo.setObjtyp("FXD");
        Fxd rec = transactionVo.getFxdgrp().getRec();
        bizInfoVo.setObjinr(rec.getInr());
        bizInfoVo.setOwnref(rec.getOwnref());
        bizInfoVo.setFxtyp(rec.getFxtyp());
        bizInfoVo.setAcc(rec.getAcc());
        bizInfoVo.setAcc2(rec.getAcc2());
        bizInfoVo.setDsp(rec.getDsp());
        bizInfoVo.setDsp2(rec.getDsp2());
        bizInfoVo.setTrdint(rec.getTrdint());
        bizInfoVo.setTrdout(rec.getTrdout());
        bizInfoVo.setTrnman(rec.getTrnman());
        bizInfoVo.setRat(rec.getRat());
        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(FxtssbStoreBo transactionVo) {
        return null;
    }

    @Override
    public FxtssbStoreBo init(FxtssbStoreBo fxtssbStoreBo) {
        fxtssbStoreBo.reset();
        init950(fxtssbStoreBo);
        init1000(fxtssbStoreBo);
        return fxtssbStoreBo;
    }

    /**
     * source:fxtssb.0001.script
     */
    public void init1000(FxtssbStoreBo bo) {

        bo.getFxdgrp().getCbs().getNom1().setCur(xrtmodService.sysiso());
        // MAXAMT = 200000.00
//        Systems.setValues(fxtssbStoreVo.getFxdgrp().getRec(), "fxtyp", "SB");
        //结售汇类型  SB:即期结汇
        bo.getFxdgrp().getRec().setFxtyp("SB");
        String extkey = SettleContext.getUserSession().getUsr().getExtkey();
        bo.getFxdgrp().getRec().setUsr(extkey);
        bo.getFxdgrp().getRec().setOwnusr(extkey);
        //钞汇 默认 1:现汇
        bo.getFxtp().setCshflg("1");
        //这笔结售汇业务的成交日期，系统默认为当前日期，可以根据业务实际情况修改。
        bo.getFxdgrp().getRec().setOpndat(Dates.today());
        //这笔结售汇业务的起息日，系统默认为成交日，可以根据业务实际情况修改。
        bo.getFxdgrp().getRec().setValdat(bo.getFxdgrp().getRec().getOpndat());
        //换算外币
        bo.getFxtp().setFrgchk("X");

    }

    private void init950(FxtssbStoreBo fxtssbStoreBo) {
    }

    @Override
    public void beforeSave(FxtssbStoreBo transactionVo) {
        transactionVo.reset();
        Fxdgrp fxdgrp = transactionVo.getFxdgrp();
        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(fxdgrp.getRec());
        transactionServiceSet.setBlk(fxdgrp.getBlk());
        transactionServiceSet.setTransactionVo(transactionVo);
        fxToolComponent.collectCbsMap(transactionServiceSet, "", fxdgrp.getCbs(), "max", "nom1", "opn1", "opn2");
        fxToolComponent.collectPtsList(transactionServiceSet, fxdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());
    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, FxtssbStoreBo transactionVo) {
        bizInfoVo.setObjinr(transactionVo.getFxdgrp().getRec().getInr());
    }


    /**
     * source:fxtssb.0003.script
     */
    public void cbsSav(FxtssbStoreBo transactionVo, TrnmodVo trnmodVo) {
        cbsmodService.cbsTrnSetNew("TRN", trnmodVo.getTrn().getInr());
        Fxacbs cbs = transactionVo.getFxdgrp().getCbs();
        String trninr = transactionHelpService.getTrninr();
        trnmodService.cbsTrnStore("MAXAMT", "AMT1", Dates.today(), cbs.getMax().getCur(), cbs.getMax().getAmt(), I18nUtil.getMessage("funds_fxtssb_000000"), transactionVo.getFxdgrp().getRec(), trninr);
        if (!MdaUtils.isEmpty(cbs.getNom1().getAmt())) {
            trnmodService.cbsTrnStore("NOMAMT", "AMT1", Dates.today(), cbs.getNom1().getCur(), cbs.getNom1().getAmt(), I18nUtil.getMessage("funds_fxtssb_000000"), transactionVo.getFxdgrp().getRec(), trninr);
            trnmodService.cbsTrnStore("UTLAMT", "AMT1", Dates.today(), cbs.getMax().getCur(), cbs.getMax().getAmt(), I18nUtil.getMessage("funds_fxtssb_000000"), transactionVo.getFxdgrp().getRec(), trninr);
        }
    }

    /**
     * source:fxtssb.0022.script
     */
    public Map<String, List<String>> defaultAccountsN1000(FxtssbStoreVo fxtssbStoreVo) {
        fxtssbStoreVo.reset();
        String trnTyp = NULLSTR;
        String accLst = NULLSTR;
        IStream accStm = new StreamImpl();
        String acc2Lst = NULLSTR;
        IStream acc2Stm = new StreamImpl();
        Map<String, List<String>> map = new HashMap();
        if (!MdaUtils.isEmpty(fxtssbStoreVo.getFxdgrp().getCbs().getMax().getCur()) && !MdaUtils.isEmpty(fxtssbStoreVo.getFxdgrp().getApl().getPts().getPtyinr())) {
            if (!MdaUtils.isEmpty(fxtssbStoreVo.getFxdgrp().getRec().getDsp())) {
                trnTyp = fxtssbStoreVo.getFxdgrp().getRec().getDsp();
                if (MdaUtils.compareTo(trnTyp, "CA") == 0) {
                    trnTyp = "LO-TT2";
                }
                if (MdaUtils.compareTo(trnTyp, "SP") == 0 || MdaUtils.compareTo(trnTyp, "HA") == 0) {
                    accLst = glemodService.getAvailAcc(trnTyp, fxtssbStoreVo.getFxdgrp().getCbs().getMax().getCur(), "", fxtssbStoreVo.getFxdgrp().getCbs().getMax().getAmt());
                } else {
                    accLst = glemodService.getAvailAcc(trnTyp, fxtssbStoreVo.getFxdgrp().getCbs().getMax().getCur(), fxtssbStoreVo.getFxdgrp().getApl().getPts().getPtyinr(), fxtssbStoreVo.getFxdgrp().getCbs().getMax().getAmt());
                }
//                Systems.setValues(fxtssbStoreVo.getFxdgrp().getRec(), "acc", accLst);
                if (StringUtils.isNotEmpty(accLst)) {
                    String[] split = accLst.split("[\\n]|\\r\\n");
                    map.put("accLst", new ArrayList<>(Arrays.asList(split)));
                }
                if (!fxtssbStoreVo.getFxdgrp().getRec().isModified("acc")) {
                    if (!MdaUtils.isEmpty(accLst)) {
                        Streams.streamClear(accStm);
                        Streams.streamSet(accStm, accLst);
                        fxtssbStoreVo.getFxdgrp().getRec().setAcc(Strings.getLine(accStm, 1));
                    }
                }
//                Systems.setEnabled(fxtssbStoreVo.getFxdgrp().getRec(), "acc");
            } else {
//                Dynamic.disableAndClearField(MdaDriver.getDatafield(fxtssbStoreVo.getFxdgrp().getRec(), "acc"));
                fxtssbStoreVo.getFxdgrp().getRec().setAcc(NULLSTR);
            }
            if (!MdaUtils.isEmpty(fxtssbStoreVo.getFxdgrp().getRec().getDsp2())) {
                if (MdaUtils.compareTo(fxtssbStoreVo.getFxdgrp().getRec().getDsp2(), "SP") == 0) {
                    acc2Lst = glemodService.getAvailAcc(fxtssbStoreVo.getFxdgrp().getRec().getDsp2(), "CNY", "", BigDecimal.ONE);
                } else {
                    acc2Lst = glemodService.getAvailAcc(fxtssbStoreVo.getFxdgrp().getRec().getDsp2(), "CNY", fxtssbStoreVo.getFxdgrp().getApl().getPts().getPtyinr(), BigDecimal.ONE);
                }
//                Systems.setValues(fxtssbStoreVo.getFxdgrp().getRec(), "acc2", acc2Lst);
                if (StringUtils.isNotEmpty(acc2Lst)) {
                    String[] split = acc2Lst.split("[\\n]|\\r\\n");
                    map.put("acc2Lst", new ArrayList<>(Arrays.asList(split)));
                }
                if (!fxtssbStoreVo.getFxdgrp().getRec().isModified("acc2")) {
                    if (!MdaUtils.isEmpty(acc2Lst)) {
                        Streams.streamClear(acc2Stm);
                        Streams.streamSet(acc2Stm, acc2Lst);
                        fxtssbStoreVo.getFxdgrp().getRec().setAcc2(Strings.getLine(acc2Stm, 1));
                    }
                }
//                Systems.setEnabled(fxtssbStoreVo.getFxdgrp().getRec(), "acc2");
            } else {
//                Dynamic.disableAndClearField(MdaDriver.getDatafield(fxtssbStoreVo.getFxdgrp().getRec(), "acc2"));
                fxtssbStoreVo.getFxdgrp().getRec().setAcc2(NULLSTR);
            }
        }
        return map;
    }

    public String getRef(FxtssbStoreVo fxtssbStoreVo) {
        fxtssbStoreVo.reset();
        PtsptaVo ptsptaVo = fxtssbStoreVo.getFxdgrp().getApl();
        if (ptsptaVo == null || StringUtils.isEmpty(ptsptaVo.getPtainr())) {
            return NULLSTR;
        }
        Pta pta = ptaCacheService.getPtaByInr(ptsptaVo.getPtainr());
        Bch wrkbch = new Bch();
        wrkbch.setInr(pta.getBranchinr());
        wrkbch = bchService.selectByPrimaryKey(wrkbch);
        if (Objects.isNull(wrkbch)) {
            return NULLSTR;
        }
        SettleContext.getUserSession().setWrkbch(wrkbch);
        Fxdgrp fxdgrp = fxtssbStoreVo.getFxdgrp();
        fxToolComponent.getref(fxdgrp);
        return fxdgrp.getRec().getOwnref();
    }


    /**
     * source:fxtssb.0005.script
     * fxdgrp.cbs.max.cur 外币币种
     * fxdgrp.apl.pts.ptyinr 申请人
     * fxdgrp.rec.fxtyp 结售汇类型  SB：即期结售汇
     * fxtp.cshflg 钞汇标志  1:现汇  2：现钞
     * fxtp.quoflg  Quote Flag
     */
    public void defaultRateN1000(FxtssbStoreVo vo) {
        vo.reset();
        String rmbRat = NULLSTR;
        BigDecimal buyRat = null;
        BigDecimal selRat = null;
        BigDecimal midRat = null;
        BigDecimal buy1Rat = null;
        String selTyp = NULLSTR;
        BigDecimal selPreRat = null;
        String buyTyp = NULLSTR;
        BigDecimal buyPreRat = null;
        String setCur = NULLSTR;
        BigDecimal setAmt = BigDecimal.ZERO;
        BigDecimal favRat = BigDecimal.ZERO;
        Cbb max = vo.getFxdgrp().getCbs().getMax();
        Fxd rec = vo.getFxdgrp().getRec();
        Pts pts = vo.getFxdgrp().getApl().getPts();
        if (MdaUtils.isEmpty(vo.getFxtp().getQuoflg())
                && !MdaUtils.isEmpty(rec.getFxtyp()) && !MdaUtils.isEmpty(pts.getPtyinr())) {
//            Systems.setEnabled(vo.getFxdgrp().getRec(), "rat");
            Argument<String> rmbRatBox = new Argument<String>();
            Argument<BigDecimal> buyRatBox = new Argument<BigDecimal>();
            Argument<BigDecimal> selRatBox = new Argument<BigDecimal>();
            Argument<BigDecimal> midRatBox = new Argument<BigDecimal>();
            Argument<BigDecimal> buy1RatBox = new Argument<BigDecimal>();
            xrtmodService.getratCebTim(max.getCur(), xrtmodService.sysiso(), Dates.today(), rmbRatBox, buyRatBox, selRatBox, midRatBox, buy1RatBox);
            rmbRat = rmbRatBox.value;
            buyRat = buyRatBox.value;
            selRat = selRatBox.value;
            midRat = midRatBox.value;
            buy1Rat = buy1RatBox.value;
            Ptyrat ptyrat = ptyratMapper.getPtyratByPtyinrAndCur(pts.getPtyinr(), max.getCur());
            if (Objects.nonNull(ptyrat)) {
                selTyp = ptyrat.getSeltyp();
                selPreRat = ptyrat.getBuyrat();
                buyTyp = ptyrat.getBuytyp();
                buyPreRat = ptyrat.getBuyrat();
            }
            // 如果没有优惠，则根据配置文件来进行优惠
            setCur = max.getCur();
            SetmodVo setmodVo = buildSetmodVo(vo);
            setAmt = setmodService.getSettlementAmount(setmodVo, setCur);
            // 默认都是按照点数优惠
            if (MdaUtils.compareTo(setAmt, 0) > 0) {
                buyTyp = "1";
                selTyp = "1";
               /* TODO setusd.ini  未提供 暂时注释
               favRat = vo.getFxtp().getFavorRatePoint(setCur, setAmt);
                if (MdaUtils.compareTo(favRat, buyPreRat) > 0) {
                    buyPreRat = favRat;
                }
                if (MdaUtils.compareTo(favRat, selPreRat) > 0) {
                    selPreRat = favRat;
                }*/
            }
            if (!rec.isModified("rat")) {
                if (MdaUtils.compareTo(Strings.mid(rec.getFxtyp(), 2, 1), "B") == 0) {
                    if (MdaUtils.compareTo(vo.getFxtp().getCshflg(), "2") == 0) {
                        rec.setRat(buy1Rat);
                    } else {
                        switch (buyTyp) {
                            case "1":
                                rec.setRat(Decimals.add(buyRat, Decimals.div(buyPreRat, 10000)));
                                break;
                            case "2":
                                rec.setRat(Decimals.add(buyRat, Decimals.div(Decimals.mul(buyRat, buyPreRat), 100)));
                                break;
                            default:
                                rec.setRat(buyRat);
                        }
                    }
                } else {
                    switch (Strings.trim(selTyp)) {
                        case "1":
                            rec.setRat(Decimals.sub(selRat, Decimals.div(selPreRat, 10000)));
                            break;
                        case "2":
                            rec.setRat(Decimals.sub(selRat, Decimals.div(Decimals.mul(selRat, selPreRat), 100)));
                            break;
                        default:
                            rec.setRat(selRat);
                    }
                }
            }
            rec.setRat(Numerics.round(rec.getRat(), 4));
            rec.setMidrat(Numerics.round(midRat, 4));
        } else {
//            Systems.resetEnabled(rec, "rat");
            rec.setRat(BigDecimal.ZERO);
            rec.setMidrat(BigDecimal.ZERO);
        }
    }

    private SetmodVo buildSetmodVo(FxtssbStoreVo vo) {
        SetmodVo setmodVo = new SetmodVo();
        Map cbsMap = new HashMap();
        Cbb max = vo.getFxdgrp().getCbs().getMax();
        cbsMap.put("MAX", max);
        setmodVo.setCbsMap(cbsMap);
        List<PtsptaVo> ptsList = new ArrayList();
        PtsptaVo apl = vo.getFxdgrp().getApl();
        ptsList.add(apl);
        setmodVo.setPtsList(ptsList);
        FxdBizInfoVo bizInfoVo = new FxdBizInfoVo();
        bizInfoVo.setObjtyp("FXD");
        setmodVo.setRec(bizInfoVo);
        return setmodVo;
    }

    public void defaultCalculateN1100(FxtssbStoreVo fxtssbStoreVo) {
        fxtssbStoreVo.reset();
        fxtpService.defaultCalculateN1100(fxtssbStoreVo.getFxdgrp(), fxtssbStoreVo.getFxtp());
    }
}
